package at.jku.ssw.graphviz;

import java.util.LinkedList;

/** A graph of character vertices. */
public class Graph {

	public final LinkedList<Vertex> vertices;
	public final LinkedList<Edge> edges;

	/** Creates an empty graph. */
	public Graph() {
		vertices = new LinkedList<>();
		edges = new LinkedList<>();
	}

	/** Adds a vertex to the graph. */
	public void addVertex(Vertex vertex) {
		for (int i = 0; i < vertices.size(); i++) {
			Vertex v = (Vertex) vertices.get(i);
			if (vertex.value == v.value) {
				throw new IllegalArgumentException("Vertex already exists");
			}
		}
		if (vertex.isRoot && getRoot() != null) {
			throw new IllegalArgumentException("Root already exists");
		}
		vertices.addLast(vertex);
	}

	/** Returns the root vertex of the graph. */
	public Vertex getRoot() {
		for (int i = 0; i < vertices.size(); i++) {
			Vertex v = (Vertex) vertices.get(i);
			if (v.isRoot) {
				return v;
			}
		}
		return null;
	}

	/** Adds an edge to the graph. */
	public void addEdge(Edge edge) {
		checkVertex(edge.start);
		checkVertex(edge.end);
		for (int i = 0; i < edges.size(); i++) {
			Edge e = (Edge) edges.get(i);
			if (isEqual(e, edge)) {
				throw new IllegalArgumentException("Edge already exists");
			}
		}
		edges.addLast(edge);
	}

	private boolean isEqual(Edge e1, Edge e2) {
		return (e1.start == e2.start && e1.end == e2.end) || ((e1.start == e2.end && e1.end == e2.start
				&& (e1.kind == Edge.Kind.Undirected || e2.kind == Edge.Kind.Undirected)));
	}

	private void checkVertex(Vertex vertex) {
		for (int i = 0; i < vertices.size(); i++) {
			Vertex v = (Vertex) vertices.get(i);
			if (v == vertex) {
				return;
			}
		}
		throw new IllegalArgumentException("Vertex does not exist");
	}

	/** Returns the outgoing edges for a vertex. */
	public Edge[] getEdges(Vertex vertex) {
		LinkedList<Edge> result = new LinkedList<>();
		for (int i = 0; i < edges.size(); i++) {
			Edge e = (Edge) edges.get(i);
			if (e.start == vertex) {
				result.addLast(e);
			} else if (e.kind == Edge.Kind.Undirected && e.end == vertex) {
				result.addLast(new Edge(e.end, e.start, e.weight, e.kind));
			}
		}
		return (Edge[]) result.toArray(new Edge[0]);
	}

	/** Returns the edge between the start and the end vertex. */
	public Edge getEdge(Vertex start, Vertex end) {
		for (int i = 0; i < edges.size(); i++) {
			Edge e = (Edge) edges.get(i);
			if (isEqual(e, new Edge(start, end, Edge.Kind.Directed))) {
				return e;
			}
		}
		return null;
	}

}
