package at.jku.ssw.list;

import at.jku.ssw.Iterator;

/** A list of objects. */
public abstract class List {
	/**
	 * Inserts the value at position index.
	 *
	 * @throws IndexOutOfBoundsException
	 *             if the index is out of range (index < 0 || index > size())
	 */
	public abstract void insert(int index, Object value);

	/**
	 * Inserts the value at the beginning of this list.
	 */
	public abstract void insertFirst(Object value);

	/**
	 * Returns the value at position index.
	 *
	 * @throws IndexOutOfBoundsException
	 *             if the index is out of range (index < 0 || index >= size())
	 */
	public abstract Object get(int index);

	/**
	 * Removes the value at position index (optional operation).
	 *
	 * @throws IndexOutOfBoundsException
	 *             if the index is out of range (index < 0 || index >= size())
	 */
	public abstract Object remove(int index);

	/**
	 * Removes the first value of the list.
	 *
	 * @throws IndexOutOfBoundsException
	 *             if the list is empty
	 */
	public abstract Object removeFirst();

	/**
	 * Returns the index of the first occurrence of the value. If no such value
	 * occurs in this list, then -1 is returned.
	 */
	public abstract int indexOf(Object value);

	/**
	 * Returns the index of the last occurrence of the value. If no such value
	 * occurs in this list, then -1 is returned.
	 */
	public abstract int lastIndexOf(Object value);

	/**
	 * Returns the number of values in this list.
	 */
	public abstract int size();

	/**
	 * Returns an iterator of the values in this list.
	 */
	public abstract Iterator iterator();

}